#include "u3.h"

void normalChecksum(uint8 *b, int n){
  /* Add checksum to a data packet for normal command format */
  b[0]=normalChecksum8(b,n);
}


void extendedChecksum(uint8 *b, int n){
  /* Add checksum to a data packet for extended command format */

  uint16 a;

  a = extendedChecksum16(b,n);
  b[4] = (uint8)(a & 0xff);
  b[5] = (uint8)((a >> 8) & 0xff);
  b[0] = extendedChecksum8(b);
}

uint8 normalChecksum8(uint8 *b, int n){
  /* Sum bytes 1 to n-1 as uint16. Sum quotient and remainder of 256
     division. Again, sum quotient and remainder of 256
     division. Return result as uint8.  */
  int i;
  uint16 a, bb;
  
  for(i = 1, a = 0; i < n; i++) 
	a+=(uint16)b[i];

  bb=a/256;
  a=(a-256*bb)+bb;
  bb=a/256;

  return (uint8)((a-256*bb)+bb);

}


uint16 extendedChecksum16(uint8 *b, int n){
  /* Sum bytes 6 to n-1 as uint16. */

  int i,a = 0;

  for(i = 6; i < n; i++)
    a += (uint16)b[i];

  return a;
}


uint8 extendedChecksum8(uint8 *b){
  /* Sum bytes 1 to 5. Sum quotient and remainder of 256
     division. Again, sum quotient and remainder of 256
     division. Return result as uint8. */
  int i,a,bb;

  for(i=1,a=0;i<6;i++)
    a+=(uint16)b[i];
  
  bb=a/256;
  a=(a-256*bb)+bb;
  bb=a/256;
  
  return (uint8)((a-256*bb)+bb);  
}


int openUSBConnection(char *pathname) {
  /* Opens a USB device. Returns a value <0 on failure. Returns the file
     descriptor on success. */

  int fd;
  
  fd = open(pathname, O_RDWR);

  if(fd == -1)
    printf("Error: Could not open the device\n");
    
  return fd;
}


int closeUSBConnection(int fd) {

  return close(fd);
}

long getTickCount() {
/* retrieves the number of milliseconds that has elasped 
   since the system was started */

  struct timeval tv;

  gettimeofday(&tv, NULL);

  return (tv.tv_sec * 1000) + (tv.tv_usec / 1000);
}

long binaryToCalibratedAnalogVoltage(u3CalibrationInfo *caliInfo, uint8 negChannel, uint16 bytesVoltage, double *analogVoltage) {
/*  Translates the binary analog input bytes read from the u3, to a voltage 
    value (calibrated).  Call getCalibrationInfo first to set up caliInfo.  
    Returns -1 on error, 0 on success.  */
  
  if( (negChannel >= 0 && negChannel <= 15)  || negChannel == 30)
  {
    *analogVoltage = caliInfo->ainDiffSlope*((double)bytesVoltage) + caliInfo->ainDiffOffset;
  }  
  else if(negChannel == 31)
  {
    *analogVoltage = caliInfo->ainSESlope*((double)bytesVoltage) + caliInfo->ainSEOffset;
  }
//  else if(negChannel == 30)  
//  {
//    *analogVoltage = caliInfo->ainDiffSlope*((short)(bytesVoltage - 0x8000)) + caliInfo->ainDiffOffset + caliInfo->vref;
//  }
  else
  {
    printf("binaryToCalibratedAnalogVoltage error: invalid negative channel.\n");
    return -1;
  }

  return 0;  
}

long binaryToCalibratedAnalogTemperature(u3CalibrationInfo *caliInfo, uint16 bytesTemperature, double *analogTemperature) {
/*  Translates the binary analog bytes read from the u3, to a Kelvin 
    temperature value (calibrated).  Call getCalibrationInfo first to 
    set up caliInfo.  Returns -1 on error, 0 on success.  */
  
  *analogTemperature = caliInfo->tempSlope*((double)bytesTemperature);
  return 0;
}

long analogToCalibratedBinaryVoltage(u3CalibrationInfo *caliInfo, int channelNumber, double analogVoltage, uint8 *bytesVoltage) {
/*  Translates a voltage value to binary analog input bytes (calibrated) 
    that can be sent to a u3.  Call getCalibrationInfo first to set up 
    caliInfo.  Returns -1 on error, 0 on success. */

  double tempBytesVoltage;
  
  tempBytesVoltage = (analogVoltage/4.95)*256;
  
  switch(channelNumber) 
  {
    case 0:
      tempBytesVoltage = analogVoltage*caliInfo->dacSlope[0] + caliInfo->dacOffset[0];
      break;
    case 1:
      tempBytesVoltage = analogVoltage*caliInfo->dacSlope[1] + caliInfo->dacOffset[1];      
      break;
    default:
      printf("analogToCalibratedBinaryVoltage error: invalid channelNumber.\n");
      return -1;
  }	
  //Checking to make sure bytesVoltage will be a value between 0 and 255, 
  //Too high of an analogVoltage (about 4.95 and above volts) or too low 
  //(below 0 volts) will cause a value not between 0 and 255.
  if(tempBytesVoltage < 0)
    tempBytesVoltage = 0;
  if(tempBytesVoltage > 255)  
    tempBytesVoltage = 255;
  
  *bytesVoltage = (uint8)tempBytesVoltage;

  return 0;
}
		
long binaryToUncalibratedAnalogVoltage(uint8 negChannel, uint16 bytesVoltage, double *analogVoltage) {
/*  Translates the binary analog input bytes read from the u3, to 
    a voltage value (uncalibrated) */

  if( (negChannel >= 0 && negChannel <= 15) || negChannel == 30)
  {
    *analogVoltage = (((double)bytesVoltage)/65536.0)*4.88 - 4.88/2;
  }  
  else if(negChannel == 31)
  {
    *analogVoltage = (((double)bytesVoltage)/65536.0)*2.44;
  }
  //else if(negChannel == 31)  
  //{
  //  *analogVoltage = caliInfo->ainDiffSlope*((short)(bytesVoltage - 0x8000)) + caliInfo->ainDiffOffset + caliInfo->vref;
  //}
  else
  {
    printf("binaryToCalibratedAnalogVoltage error: invalid negative channel.\n");
    return -1;
  }

  return 0;
}

long analogToUncalibratedBinaryVoltage(double analogVoltage, uint8 *bytesVoltage){
/*  Translates a voltage value to binary analog input bytes (uncalibrated) 
    that can be sent to a u3.  Returns -1 on error, 0 on success. */

  double tempBytesVoltage;
  
  tempBytesVoltage = (analogVoltage/4.95)*256;
  
  //Checking to make sure bytesVoltage will be a value between 0 and 255, 
  //Too high of an analogVoltage (about 4.95 and above volts) or too low 
  //(below 0 volts) will cause a value not between 0 and 255.
  if(tempBytesVoltage < 0)
    tempBytesVoltage = 0;
  if(tempBytesVoltage > 255)  
    tempBytesVoltage = 255;
  
  *bytesVoltage = (uint8)tempBytesVoltage;

  return 0;
}

double FPuint8ArrayToFPDouble(uint8 *buffer, int startIndex) { 
/*  Converts a fixed point byte array (starting a startIndex) to 
    a floating point double value  */
    
  unsigned int resultDec;
  int resultWh;
  int i;

  resultDec = 0;
  resultWh = 0;

  for(i = 0; i < 4; i++) 
  {
    resultDec += (buffer[startIndex + i] << (i*8)); 			
    resultWh += (buffer[startIndex + i + 4] << (i*8));
  }

  return ( (double)resultWh + (double)(resultDec)/4294967296.0 );
}

long getCalibrationInfo(int fd, u3CalibrationInfo *caliInfo) {
/*  Gets calibration information from memory blocks 0-2 of a U3 
    and stores the information into a calibrationInfo structure. */
   
  uint8 sendBuffer[8];
  uint8 recBuffer[40];
  int sentRec = 0;
  
  //////////////////////////////
  //reading block 0 from memory 
  //////////////////////////////
  sendBuffer[1] = (uint8)(0xF8);  //command byte
  sendBuffer[2] = (uint8)(0x01);  //number of data words
  sendBuffer[3] = (uint8)(0x2D);  //extended command number
  sendBuffer[6] = (uint8)(0x00);
  sendBuffer[7] = (uint8)(0x00);  //Blocknum = 0
  extendedChecksum(sendBuffer, 8);
  
  sentRec = write(fd, sendBuffer, 8);
  if(sentRec < 8)
  {
    if(sentRec == -1)
      goto writeError0;
    else  
      goto writeError1;
  }	

  sentRec= read(fd, recBuffer, 40);
  if(sentRec < 40)
  {
    if(sentRec == -1)
      goto readError0;
    else  
      goto readError1;
  }

  if(recBuffer[1] != (uint8)(0xF8) || recBuffer[2] != (uint8)(0x11) || recBuffer[3] != (uint8)(0x2D))
    goto commandByteError;

  //block data starts on byte 8 of the buffer
  caliInfo->ainSESlope = FPuint8ArrayToFPDouble(recBuffer + 8, 0);
  caliInfo->ainSEOffset = FPuint8ArrayToFPDouble(recBuffer + 8, 8);
  caliInfo->ainDiffSlope = FPuint8ArrayToFPDouble(recBuffer + 8, 16);
  caliInfo->ainDiffOffset = FPuint8ArrayToFPDouble(recBuffer + 8, 24);
 
  //////////////////////////////
  //reading block 1 from memory 
  //////////////////////////////
  sendBuffer[7] = (uint8)(0x01);    //Blocknum = 1
  extendedChecksum(sendBuffer, 8);
  
  sentRec = write(fd, sendBuffer, 8);
  if(sentRec < 8)
  {
    if(sentRec == -1)
      goto writeError0;
    else  
      goto writeError1;
  }	

  sentRec= read(fd, recBuffer, 40);

  if(sentRec < 40)
  {
    if(sentRec == -1)
      goto readError0;
    else  
      goto readError1;
  }

  if(recBuffer[1] != (uint8)(0xF8) || recBuffer[2] != (uint8)(0x11) || recBuffer[3] != (uint8)(0x2D))
  {
    goto commandByteError;
  }
  
  caliInfo->dacSlope[0] = FPuint8ArrayToFPDouble(recBuffer + 8, 0);
  caliInfo->dacOffset[0] = FPuint8ArrayToFPDouble(recBuffer + 8, 8);
  caliInfo->dacSlope[1] = FPuint8ArrayToFPDouble(recBuffer + 8, 16);
  caliInfo->dacOffset[1] = FPuint8ArrayToFPDouble(recBuffer + 8, 24);

  //////////////////////////////
  //reading block 2 from memory 
  //////////////////////////////
  sendBuffer[7] = (uint8)(0x02);    //Blocknum = 2
  extendedChecksum(sendBuffer, 8);
  
  sentRec = write(fd, sendBuffer, 8);
  if(sentRec < 8)
  {
    if(sentRec == -1)
      goto writeError0;
    else  
      goto writeError1;
  }	

  sentRec= read(fd, recBuffer, 40);
  if(sentRec < 40)
  {
    if(sentRec == -1)
      goto readError0;
    else  
      goto readError1;
  }

  if(recBuffer[1] != (uint8)(0xF8) || recBuffer[2] != (uint8)(0x11) || recBuffer[3] != (uint8)(0x2D))
  {
    goto commandByteError;
  }
  
  caliInfo->tempSlope = FPuint8ArrayToFPDouble(recBuffer + 8, 0);
  caliInfo->vref = FPuint8ArrayToFPDouble(recBuffer + 8, 8);
  caliInfo->vref15 = FPuint8ArrayToFPDouble(recBuffer + 8, 16);
  caliInfo->vreg = FPuint8ArrayToFPDouble(recBuffer + 8, 24);

  return 0;
  
writeError0: 
  printf("Error : getCalibrationInfo write failed\n");
  return -1;
  
writeError1:
  printf("Error : getCalibrationInfo did not write all of the buffer\n");
  return -1;

readError0:
  printf("Error : getCalibrationInfo read failed\n");
  return -1;
  
readError1:
  printf("Error : getCalibrationInfo did not read all of the buffer\n");
  return -1;

commandByteError:
  printf("Error : getCalibrationInfo received wrong command bytes for ReadMem\n");
  return -1;
}

