//Author : LabJack
//03/06/2006
//This example will set FIO0 and FIO1 to analog input, FIO2 to digital output 
//set to low, and FIO3 to digital input.  FIO4 and FIO5 will be set as timers 
//0 and 1 with timer modes PMW8 and PMW16.  FIO6 will be set as counter 1.  
//Also, DAC0 will be set to 1.5 volts and the temperature will be read.

#include <stdio.h>
#include <termios.h>
#include "u3.h"

static struct termios termNew, termOrig;
static int peek = -1;

int configIO_example(int fd, int enable);
int configTimerClock_example(int fd);
int feedback_setup_example(int fd, u3CalibrationInfo *caliInfo);
int feedback_loop_example(int fd, u3CalibrationInfo *caliInfo);


void setTerm();
int kbhit();
void unsetTerm();

int main(int argc, char **argv)
{
  int fd;
  u3CalibrationInfo caliInfo;

  //setting terminal settings
  setTerm();

  fd = openUSBConnection("/dev/usb/labjacku3_0");

  if(fd<0)
    goto done;
  
  if(getCalibrationInfo(fd, &caliInfo) < 0)
    goto close;

  
  if(configIO_example(fd, 1) != 0)
    goto close;

  if(configTimerClock_example(fd) != 0)
    goto close;

  if(feedback_setup_example(fd, &caliInfo) != 0)
    goto close;
  
  if(feedback_loop_example(fd, &caliInfo) != 0)
    goto close;

  configIO_example(fd, 0);

close:
  if(closeUSBConnection(fd) < 0)
    printf("Error: failed to close USB connection.\n");

done:
  printf("\nDone\n");

  //Setting terminal settings to previous settings
  unsetTerm();
  return 0;
}

//Configures the FIOs, DAC, Timers and Counters for this example
int configIO_example(int fd, int enable)
{
  uint8 sendBuff[12], recBuff[12];
  uint16 checksumTotal;
  int sendChars, recChars;
  uint8 timerCounterConfig, fioAnalog;

  if(enable == 0)
  {
    timerCounterConfig = 0;  //Disabling timers (bits 0 and 1) and Counters (bits 2 and 3)
    fioAnalog = 255;         //Setting all FIOs to analog  
  }
  else
  {
    timerCounterConfig = 74;  //Enabling 2 timers (bits 0 and 1), Counter 1 (bit 3) 
                              //and setting TimerCounterPinOffset (bits 4-7) to 4.
                              //Note that Counter 0 will not be available since the 
                              //timer clock will use a divisor in this example.  
    fioAnalog = 3;  //Setting FIO0 (bit 0) and FIO1 (bit 1) to analog input  
  }			
  
  sendBuff[1] = (uint8)(0xF8);  //Command byte
  sendBuff[2] = (uint8)(0x03);  //Number of data words
  sendBuff[3] = (uint8)(0x0B);  //Extended command number

  sendBuff[6] = 5;  //Writemask : Setting writemask for timerCounterConfig (bit 0) 
                    //and FIOAnalog (bit 2)
  
  sendBuff[7] = 0;  //Reserved
  sendBuff[8] = timerCounterConfig;  //TimerCounterConfig
  sendBuff[9] = 0;  //DAC1 enable : not enabling
  sendBuff[10] = fioAnalog; //FIOAnalog
  sendBuff[11] = 0; //EIOAnalog : Not setting anything                       
  extendedChecksum(sendBuff,12);

  //Sending command to U3
  if( (sendChars = write(fd, sendBuff, 12)) < 12)
  {
    if(sendChars == -1)
      printf("ConfigIO error : write failed\n");
    else  
      printf("ConfigIO error : did not write all of the buffer\n");
    return -1;
  }

  //Reading response from U3
  if( (recChars = read(fd, recBuff, 12)) < 12)
  {
    if(recChars == -1)
      printf("ConfigIO error : read failed\n");
    else  
      printf("ConfigIO error : did not read all of the buffer\n");
    return -1;
  }

  checksumTotal = extendedChecksum16(recBuff, 12);
  if( (uint8)((checksumTotal >> 8) & 0xff) != recBuff[5])
  {
    printf("ConfigIO error : read buffer has bad checksum16(MSB)\n");
    return -1;
  }

  if( (uint8)(checksumTotal & 0xff) != recBuff[4])
  {
    printf("ConfigIO error : read buffer has bad checksum16(LBS)\n");
    return -1;
  }

  if( extendedChecksum8(recBuff) != recBuff[0])
  {
    printf("ConfigIO error : read buffer has bad checksum8\n");
    return -1;
  }

  if( recBuff[1] != (uint8)(0xF8) || recBuff[2] != (uint8)(0x03) || recBuff[3] != (uint8)(0x0B) )
  {
    printf("ConfigIO error : read buffer has wrong command bytes\n");
    return -1;
  }

  if( recBuff[6] != 0)
  {
    printf("ConfigIO error : read buffer received errorcode %d\n", recBuff[6]);
    return -1;
  }
  
  if( recBuff[8] != timerCounterConfig)
  {
    printf("ConfigIO error : TimerCounterConfig did not get set correctly\n");
    return -1;
  }

  if( recBuff[10] != fioAnalog)
  {
    printf("ConfigIO error : FIOAnalog did not set correctly\n");
    return -1;
  }

  return 0;  
}

//Configures the timer clock for this example
int configTimerClock_example(int fd)
{
  uint8 sendBuff[10], recBuff[10];
  uint16 checksumTotal;
  int sendChars, recChars;
  
  sendBuff[1] = (uint8)(0xF8);  //Command byte
  sendBuff[2] = (uint8)(0x02);  //Number of data words
  sendBuff[3] = (uint8)(0x0A);  //Extended command number

  sendBuff[6] = 0;   //Reserved
  sendBuff[7] = 0;   //Reserved
 
  sendBuff[8] = 134;  //TimerClockConfig : Configuring the clock (bit 7) and setting
                      //the TimerClockBase (bits 0-2) to 24MHz/TimerClockDivisor
  sendBuff[9] = 2;    //TimerClockDivisor : Setting to 2, so the actual timer clock is 12 MHz
  extendedChecksum(sendBuff,10);

  //Sending command to U3
  if( (sendChars = write(fd, sendBuff, 10)) < 10)
  {
    if(sendChars == -1)
      printf("ConfigTimerClock error : write failed\n");
    else  
      printf("ConfigTimerClock error : did not write all of the buffer\n");
    return -1;
  }

  //Reading response from U3
  if( (recChars = read(fd, recBuff, 10)) < 10)
  {
    if(recChars == -1)
      printf("ConfigTimerClock error : read failed\n");
    else  
      printf("ConfigTimerClock error : did not read all of the buffer\n");
    return -1;
  }

  checksumTotal = extendedChecksum16(recBuff, 10);
  if( (uint8)((checksumTotal >> 8) & 0xff) != recBuff[5])
  {
    printf("ConfigTimerClock error : read buffer has bad checksum16(MSB)\n");
    return -1;
  }

  if( (uint8)(checksumTotal & 0xff) != recBuff[4])
  {
    printf("ConfigTimerClock error : read buffer has bad checksum16(LBS)\n");
    return -1;
  }

  if( extendedChecksum8(recBuff) != recBuff[0])
  {
    printf("ConfigTimerClock error : read buffer has bad checksum8\n");
    return -1;
  }

  if( recBuff[1] != (uint8)(0xF8) || recBuff[2] != (uint8)(0x02) || recBuff[3] != (uint8)(0x0A) )
  {
    printf("ConfigTimerClock error : read buffer has wrong command bytes\n");
    return -1;
  }

  if( recBuff[6] != 0)
  {
    printf("ConfigTimerClock error : read buffer received errorcode %d\n", recBuff[6]);
    return -1;
  }

/* 
  if( recBuff[8] != timerClockConfig) 
  {
    printf("ConfigTimerClock error : TimerClockConfig did not get set correctly %d\n", recBuff[7]);
    return -1;
  }

  if( recBuff[9] != timerClockDivisor) 
  {
    printf("ConfigTimerClock error : TimerClockDivisor did not get set correctly %d\n", recBuff[7]);
    return -1;
  }
  
*/
  return 0;
}

//Configures up digital directions, states, timer modes and DAC0 for this example
int feedback_setup_example(int fd, u3CalibrationInfo *caliInfo)
{
  uint8 sendBuff[32], recBuff[18];
  int sendChars, recChars;
  uint16 checksumTotal;
  
  sendBuff[1] = (uint8)(0xF8);  //Command byte
  sendBuff[2] = 13;              //Number of data words (.5 word for echo, 8 words for IOTypes 
                                //and data, and .5 word for the extra byte)   
  sendBuff[3] = (uint8)(0x00);  //Extended command number

  sendBuff[6] = 0;     //Echo  

  sendBuff[7] = 13;    //IOType is BitDirWrite
  sendBuff[8] = 130;   //IONumber (bits 0 - 4) is 2 and Direction (bit 7) is output 

  sendBuff[9] = 13;    //IOType is BitDirWrite
  sendBuff[10] = 3;    //IONumber (bits 0 - 4) is 3 and Direction (bit 7) is input 
  
  sendBuff[11] = 11;   //IOType is BitStateWrite
  sendBuff[12] = 2;    //IONumber (bits 0 - 4) is 2 and State (bit 7) is low

  sendBuff[13] = 43;   //IOType is Timer0Config
  sendBuff[14] = 0;    //TimerMode is 16 bit PWM output (mode 0)
  sendBuff[15] = 0;    //Value LSB
  sendBuff[16] = 0;    //Value MSB, Whole value is 32768 
  
  sendBuff[17] = 42;   //IOType is Timer0
  sendBuff[18] = 1;    //UpdateReset
  sendBuff[19] = 0;    //Value LSB
  sendBuff[20] = 128;  //Value MSB, Whole Value is 32768
  
  sendBuff[21] = 45;   //IOType is Timer1Config
  sendBuff[22] = 1;    //TimerMode is 8 bit PWM output (mode 1)
  sendBuff[23] = 0;    //Value LSB
  sendBuff[24] = 0;    //Value MSB, Whole value is 32768
  
  sendBuff[25] = 44;   //IOType is Timer1
  sendBuff[26] = 1;    //UpdateReset
  sendBuff[27] = 0;    //Value LSB
  sendBuff[28] = 128;  //Value MSB, Whole Value is 32768
  
  sendBuff[29] = 34;   //IOType is DAC0
  analogToCalibratedBinaryVoltage(caliInfo, 0, 1.5, &sendBuff[30]);  //Value is 1.5 volts (in binary form)
  sendBuff[31] = 0;    //Extra byte

  extendedChecksum(sendBuff,32);

  //Sending command to U3
  if( (sendChars = write(fd, sendBuff, 32)) < 32)
  {
    if(sendChars == -1)
      printf("Feedback setup error : write failed\n");
    else  
      printf("Feedback setup error : did not write all of the buffer\n");
    return -1;
  }
  
  //Reading response from U3
  if( (recChars = read(fd, recBuff, 18)) < 18)
  {
    if(recChars == -1)
    {
      printf("Feedback setup error : read failed\n");
      return -1;
    }
    else  
      printf("Feedback setup error : did not read all of the buffer\n");
  }
  
  checksumTotal = extendedChecksum16(recBuff, 18);
  if( (uint8)((checksumTotal >> 8) & 0xff) != recBuff[5])
  {
    printf("Feedback setup error : read buffer has bad checksum16(MSB)\n");
    return -1;
  }

  if( (uint8)(checksumTotal & 0xff) != recBuff[4])
  {
    printf("Feedback setup error : read buffer has bad checksum16(LBS)\n");
    return -1;
  }

  if( extendedChecksum8(recBuff) != recBuff[0])
  {
    printf("Feedback setup error : read buffer has bad checksum8\n");
    return -1;
  }

  if( recBuff[1] != (uint8)(0xF8) || recBuff[2] != 6 || recBuff[3] != (uint8)(0x00) )
  {
    printf("Feedback setup error : read buffer has wrong command bytes \n");
    return -1;
  }

  if( recBuff[6] != 0)
  {
    printf("Feedback setup error : received errorcode %d for frame %d in Feedback response. \n", recBuff[6], recBuff[7]);
    return -1;
  }
  
  return 0;
}

int feedback_loop_example(int fd, u3CalibrationInfo *caliInfo)
{
  long count;
  uint8 sendBuff[32], recBuff[28];
  int sendChars, recChars;
  uint16 checksumTotal;
  double voltage, temperature;
 
  sendBuff[1] = (uint8)(0xF8);  //Command byte
  sendBuff[2] = 13;             //Number of data words (.5 word for echo, 12.5 words for IOTypes)   
  sendBuff[3] = (uint8)(0x00);  //Extended command number

  sendBuff[6] = 0;    //Echo  

  sendBuff[7] = 1;    //IOType is AIN
  sendBuff[8] = 0;    //Positive channel (bits 0-4) is 0, LongSettling (bit 6) is not set
                      //and QuickSample (bit 7) is not set
  sendBuff[9] = 31;   //Negative channel is 31 (SE)
  
  sendBuff[10] = 1;   //IOType is AIN
  sendBuff[11] = 1;   //Positive channel (bits 0-4) is 1, LongSettling (bit 6) is not set
                      //and QuickSample (bit 7) is not set
  sendBuff[12] = 31;  //Negative channel is 31 (SE)

  sendBuff[13] = 1;   //IOType is AIN
  sendBuff[14] = 0;   //Positive channel (bits 0-4) is 0, LongSettling (bit 6) is not set
                      //and QuickSample (bit 7) is not set
  sendBuff[15] = 1;   //Negative channel is 1 (FIO1)
  
  sendBuff[16] = 1;   //IOType is AIN
  sendBuff[17] = 1;   //Positive channel (bits 0-4) is 1, LongSettling (bit 6) is not set
                      //and QuickSample (bit 7) is not set
  sendBuff[18] = 0;   //Negative channel is 0 (FIO0)

  sendBuff[19] = 1;   //IOType is AIN
  sendBuff[20] = 0;   //Positive channel (bits 0-4) is 0, LongSettling (bit 6) is not set
                      //and QuickSample (bit 7) is not set
  sendBuff[21] = 30;  //Negative channel is 30 (Vref)
  
  sendBuff[22] = 1;   //IOType is AIN
  sendBuff[23] = 1;   //Positive channel (bits 0-4) is 1, LongSettling (bit 6) is not set
                      //and QuickSample (bit 7) is not set
  sendBuff[24] = 30;  //Negative channel is 30 (Vref)
  
  sendBuff[25] = 10;  //IOType is BitStateRead
  sendBuff[26] = 3;   //IO number is 3 (FIO3)

  sendBuff[27] = 55;  //IOType is Counter1
  sendBuff[28] = 0;   //Reset (bit 0) is not set

  sendBuff[29] = 1;   //IOType is AIN
  sendBuff[30] = 30;  //Positive channel is 30 (temp sensor)
  sendBuff[31] = 31;  //Negative channel is 31 (SE)

  extendedChecksum(sendBuff,32);

  printf("Running Feedback calls in a loop\n");
  
  count = 0;
  while(!kbhit())
  {
    count++;
    printf("Iteration %ld\n", count);

    //Sending command to U3
    if( (sendChars = write(fd, sendBuff, 32)) < 32)
    {
      if(sendChars == -1)
        printf("Feedback loop error : write failed\n");
      else  
        printf("Feedback loop error : did not write all of the buffer\n");
      return -1;
    }
  
    //Reading response from U3
    if( (recChars = read(fd, recBuff, 28)) < 28)
    {
      if(recChars == -1)
      {
        printf("Feedback loop error : read failed\n");
        return -1;
      }
      else  
        printf("Feedback loop error : did not read all of the expected buffer\n");
    }
  
    if(recChars < 10)
    {
      printf("Feedback loop error : response is not large enough\n");
      return -1;
    }
      
    checksumTotal = extendedChecksum16(recBuff, recChars);
    if( (uint8)((checksumTotal >> 8) & 0xff) != recBuff[5])
    {
      printf("Feedback loop error : read buffer has bad checksum16(MSB)\n");
      return -1;
    }

    if( (uint8)(checksumTotal & 0xff) != recBuff[4])
    {
      printf("Feedback loop error : read buffer has bad checksum16(LBS)\n");
      return -1;
    }

    if( extendedChecksum8(recBuff) != recBuff[0])
    {
      printf("Feedback loop error : read buffer has bad checksum8\n");
      return -1; 
    }

    if( recBuff[1] != (uint8)(0xF8) ||	recBuff[3] != (uint8)(0x00) )
    {
      printf("Feedback loop error : read buffer has wrong command bytes \n");
      return -1;
    }

    if( recBuff[6] != 0)
    {
      printf("Feedback loop error : received errorcode %d for frame %d ", recBuff[6], recBuff[7]); 
      switch(recBuff[7])
      {
        case 1: printf("(AIN(SE))\n"); break;
        case 2: printf("(AIN(SE))\n"); break;
        case 3: printf("(AIN(Neg. chan. 1))\n"); break;
        case 4: printf("(AIN(Neg. chan. 0))\n"); break;
        case 5: printf("(AIN(Neg. chan. Vref))\n"); break;
        case 6: printf("(AIN(Neg. chan. Vref))\n"); break;        
        case 7: printf("(BitStateRead for FIO3)\n"); break;
        case 8: printf("(Counter1)\n"); break;
        case 9: printf("(Temp. Sensor\n"); break;        
        default: printf("(Unknown)\n"); break;
      }
      return -1;
    }

    binaryToCalibratedAnalogVoltage(caliInfo, 31, recBuff[9] + recBuff[10]*256, &voltage); 
    printf("AIN0(SE) : %.3f volts\n", voltage);
    binaryToCalibratedAnalogVoltage(caliInfo, 31, recBuff[11] + recBuff[12]*256, &voltage);     
    printf("AIN1(SE) : %.3f volts\n", voltage);
    binaryToCalibratedAnalogVoltage(caliInfo, 1, recBuff[13] + recBuff[14]*256, &voltage); 
    printf("AIN0(Neg. chan. 1) : %.3f volts\n", voltage);
    binaryToCalibratedAnalogVoltage(caliInfo, 0, recBuff[15] + recBuff[16]*256, &voltage);     
    printf("AIN1(Neg. chan. 0): %.3f volts\n", voltage);
    binaryToCalibratedAnalogVoltage(caliInfo, 30, recBuff[17] + recBuff[18]*256, &voltage); 
    printf("AIN0(Neg. chan. Vref) : %.3f volts\n", voltage);
    binaryToCalibratedAnalogVoltage(caliInfo, 30, recBuff[19] + recBuff[20]*256, &voltage);     
    printf("AIN1(Neg. chan. Vref): %.3f volts\n", voltage);
    printf("FIO3 state : %d\n", recBuff[21]);
    printf("Counter1(FIO6) : %u\n\n", recBuff[22] + recBuff[23]*256 + recBuff[24]*65536 + recBuff[25]*16777216);
    binaryToCalibratedAnalogTemperature(caliInfo, recBuff[26] + recBuff[27]*256, &temperature);
    printf("Temperature : %.3f K\n\n", temperature);
    sleep(1);
  }
  return 0;
}

void setTerm()
{
  tcgetattr(0, &termOrig);
  termNew = termOrig;
  termNew.c_lflag &= ~ICANON;
  termNew.c_lflag &= ~ECHO;
  termNew.c_lflag &= ~ISIG;
  termNew.c_cc[VMIN] = 1;
  termNew.c_cc[VTIME] = 0;
  tcsetattr(0, TCSANOW, &termNew);
}

int kbhit()
{
  char ch;
  int nread;

  if(peek != -1) 
    return 1;

  termNew.c_cc[VMIN]=0;
  tcsetattr(0, TCSANOW, &termNew);
  nread = read(0,&ch,1);
  termNew.c_cc[VMIN]=1;
  tcsetattr(0, TCSANOW, &termNew);

  if(nread == 1) 
  {
    peek = ch;
    return 1;
  }

  return 0;
}

void unsetTerm()
{
  tcsetattr(0,TCSANOW, &termOrig);
}
