#include "ue9.h"

void normalChecksum(uint8 *b, int n){
  /* Add checksum to a data packet for normal command format */
  b[0]=normalChecksum8(b,n);
}


void extendedChecksum(uint8 *b, int n){
  /* Add checksum to a data packet for extended command format */

  uint16 a;

  a = extendedChecksum16(b,n);
  b[4] = (uint8)(a & 0xff);
  b[5] = (uint8)((a >> 8) & 0xff);
  b[0] = extendedChecksum8(b);
}

uint8 normalChecksum8(uint8 *b, int n){
  /* Sum bytes 1 to n-1 as uint16. Sum quotient and remainder of 256
     division. Again, sum quotient and remainder of 256
     division. Return result as uint8.  */
  int i;
  uint16 a, bb;
  
  for(i = 1, a = 0; i < n; i++) 
	a+=(uint16)b[i];

  bb=a/256;
  a=(a-256*bb)+bb;
  bb=a/256;

  return (uint8)((a-256*bb)+bb);

}


uint16 extendedChecksum16(uint8 *b, int n){
  /* Sum bytes 6 to n-1 as uint16. */

  int i,a = 0;

  for(i = 6; i < n; i++)
    a += (uint16)b[i];

  return a;
}


uint8 extendedChecksum8(uint8 *b){
  /* Sum bytes 1 to 5. Sum quotient and remainder of 256
     division. Again, sum quotient and remainder of 256
     division. Return result as uint8. */
  int i,a,bb;

  for(i=1,a=0;i<6;i++)
    a+=(uint16)b[i];
  
  bb=a/256;
  a=(a-256*bb)+bb;
  bb=a/256;
  
  return (uint8)((a-256*bb)+bb);  
}


int openUSBConnection(char *pathname) {
  /* Opens a USB device. Returns a value <0 on failure. Returns the file
     descriptor on success. */

  int fd;
  
  fd = open(pathname, O_RDWR);

  if(fd == -1)
    printf("Error: Could not open the device\n");
    
  return fd;
}


int closeUSBConnection(int fd) {

  return close(fd);
}

long getTickCount() {
/* retrieves the number of milliseconds that has elasped 
   since the system was started */

  struct timeval tv;

  gettimeofday(&tv, NULL);

  return (tv.tv_sec * 1000) + (tv.tv_usec / 1000);
}

long binaryToCalibratedAnalogVoltage(calibrationInfo *caliInfo, uint8 gainBip, uint16 bytesVoltage, double *analogVoltage) {
/*  Translates the binary analog input bytes read from the ue9, to 
    a voltage value (calibrated).  Call getCalibrationInfo first to 
    set up caliInfo.  Returns -1 on error, 0 on success.  */
    
  double slope;
  double offset;

  switch( (unsigned int)gainBip )
  {
    case 0: 
      slope = caliInfo->unipolarSlope[0];  
      offset = caliInfo->unipolarOffset[0];
      break;
    case 1: 
      slope = caliInfo->unipolarSlope[1];  
      offset = caliInfo->unipolarOffset[1];
      break;
    case 2: 
      slope = caliInfo->unipolarSlope[2];  
      offset = caliInfo->unipolarOffset[2];
      break;
    case 3: 
      slope = caliInfo->unipolarSlope[3];  
      offset = caliInfo->unipolarOffset[3];
      break;
    case 8: 
      slope = caliInfo->bipolarSlope;  
      offset = caliInfo->bipolarOffset;
      break;
    default:  
      printf("binaryToCalibratedAnalogVoltage error: invalid GainBip.\n");
      return -1;
  }
  
  *analogVoltage = (slope*bytesVoltage) + offset;
  
  return 0;  
}

long binaryToCalibratedAnalogTemperature(calibrationInfo *caliInfo, int powerLevel, uint16 bytesTemperature, double *analogTemperature) {
/*  Translates the binary analog bytes read from the ue9, to a Kelvin 
    temperature value (calibrated).  Call getCalibrationInfo first to 
    set up caliInfo.  Returns -1 on error, 0 on success.  */
  
  double slope = 0;
 
  switch( (unsigned int)powerLevel )
  {
    case 0:     //high power 
      slope = caliInfo->tempSlope;  
      break;
    case 1:     //low power
      slope = caliInfo->tempSlopeLow;
      break;
    default:  
      printf("binaryToCalibratedAnalogTemperature error: invalid powerLevel.\n");
      return -1;
  }
  
  *analogTemperature = (double)(bytesTemperature)*slope;
  return 0;
}

long analogToCalibratedBinaryVoltage(calibrationInfo *caliInfo, int channelNumber, double analogVoltage, uint16 *bytesVoltage) {
/*  Translates a voltage value to binary analog input bytes (calibrated) 
    that can be sent to a ue9.  Call getCalibrationInfo first to set up 
    caliInfo.  Returns -1 on error, 0 on success.  */

  double slope;
  double offset;
  double tempBytesVoltage;
  
  switch(channelNumber) 
  {
    case 0:
      slope = caliInfo->DACSlope[0];
      offset = caliInfo->DACOffset[0];
      break;
    case 1:
      slope = caliInfo->DACSlope[1];
      offset = caliInfo->DACOffset[1];
      break;
    default:
      printf("analogToCalibratedBinaryVoltage error: invalid DACNumber.\n");
      return -1;
  }	    
  
  tempBytesVoltage = slope*analogVoltage + offset;
  
  //Checking to make sure bytesVoltage will be a value between 0 and 4095, 
  //or that a uint16 overflow does not occur.  A too high analogVoltage 
  //(above 5 volts) or too low analogVoltage (below 0 volts) will cause a 
  //value not between 0 and 4095.
  if(tempBytesVoltage < 0)
    tempBytesVoltage = 0;
  if(tempBytesVoltage > 4095)  
    tempBytesVoltage = 4095;
  
  *bytesVoltage = (uint16)tempBytesVoltage; 
     
  return 0;
}
		
long binaryToUncalibratedAnalogVoltage(uint8 gainBip, uint16 bytesVoltage, double *analogVoltage) {
/*  Translates the binary analog input bytes read from the ue9, to 
    a voltage value (uncalibrated)  */

  switch( (unsigned int)gainBip )
  {
    case 0: 
      *analogVoltage = ((double)bytesVoltage/65536.0)*5.08;
      break;
    case 1: 
      *analogVoltage = ((double)bytesVoltage/65536.0)*2.54;
      break;
    case 2: 
      *analogVoltage = ((double)bytesVoltage/65536.0)*1.27;
      break;
    case 3: 
      *analogVoltage = ((double)bytesVoltage/65536.0)*0.63;
      break;
    case 8: 
      *analogVoltage = ((double)bytesVoltage/65536.0)*10.25;
      break;
    default:  
      printf("binaryToUncalibratedAnalogVoltage error: invalid GainBip.\n");
      return -1;
  }
  return 0;
}

long analogToUncalibratedBinaryVoltage(double analogVoltage, uint16 *bytesVoltage){
/*  Translates a voltage value to binary analog input bytes (uncalibrated) 
    that can be sent to a ue9.  Returns -1 on error, 0 on success.  */

  double tempBytesVoltage;
  
  tempBytesVoltage = (analogVoltage/4.86)*4096;
  
  //Checking to make sure bytesVoltage will be a value between 0 and 4095, 
  //or that a uint16 overflow does not occur.  A too high analogVoltage 
  //(above 5 volts) or too low analogVoltage (below 0 volts) will cause a 
  //value not between 0 and 4095.
  if(tempBytesVoltage < 0)
    tempBytesVoltage = 0;
  if(tempBytesVoltage > 4095)  
    tempBytesVoltage = 4095;
  
  *bytesVoltage = (uint16)tempBytesVoltage;

  return 0;
}

double FPuint8ArrayToFPDouble(uint8 *buffer, int startIndex) { 
/*  Converts a fixed point byte array (starting a startIndex) to 
    a floating point double value  */
    
  unsigned int resultDec;
  int resultWh;
  int i;

  resultDec = 0;
  resultWh = 0;

  for(i = 0; i < 4; i++) 
  {
    resultDec += (buffer[startIndex + i] << (i*8)); 			
    resultWh += (buffer[startIndex + i + 4] << (i*8));
  }

  return ( (double)resultWh + (double)(resultDec)/4294967296.0 );
}

long getCalibrationInfo(int fd, calibrationInfo *caliInfo) {
/*  Gets calibration information from memory blocks 0-2 of a UE9 
    and stores the information into a calibrationInfo structure. */
   
  uint8 sendBuffer[8];
  uint8 recBuffer[136];
  int sentRec = 0;
  
  //////////////////////////////
  //reading block 0 from memory 
  //////////////////////////////
  sendBuffer[1] = (uint8)(0xF8);  //command byte
  sendBuffer[2] = (uint8)(0x01);  //number of data words
  sendBuffer[3] = (uint8)(0x2A);  //extended command number
  sendBuffer[6] = (uint8)(0x00);
  sendBuffer[7] = (uint8)(0x00);  //Blocknum = 0
  extendedChecksum(sendBuffer, 8);
  
  sentRec = write(fd, sendBuffer, 8);
  if(sentRec < 8)
  {
    if(sentRec == -1)
      goto writeError0;
    else  
      goto writeError1;
  }	

  sentRec= read(fd, recBuffer, 136);
  if(sentRec < 136)
  {
    if(sentRec == -1)
      goto readError0;
    else  
      goto readError1;
  }

  if(recBuffer[1] != (uint8)(0xF8) || recBuffer[2] != (uint8)(0x41) || recBuffer[3] != (uint8)(0x2A))
    goto commandByteError;

  //block data starts on byte 8 of the buffer
  caliInfo->unipolarSlope[0] = FPuint8ArrayToFPDouble(recBuffer + 8, 0);
  caliInfo->unipolarOffset[0] = FPuint8ArrayToFPDouble(recBuffer + 8, 8);
  caliInfo->unipolarSlope[1] = FPuint8ArrayToFPDouble(recBuffer + 8, 16);
  caliInfo->unipolarOffset[1] = FPuint8ArrayToFPDouble(recBuffer + 8, 24);
  caliInfo->unipolarSlope[2] = FPuint8ArrayToFPDouble(recBuffer + 8, 32);
  caliInfo->unipolarOffset[2] = FPuint8ArrayToFPDouble(recBuffer + 8, 40);
  caliInfo->unipolarSlope[3] = FPuint8ArrayToFPDouble(recBuffer + 8, 48);
  caliInfo->unipolarOffset[3] = FPuint8ArrayToFPDouble(recBuffer + 8, 56);
  
  //////////////////////////////
  //reading block 1 from memory 
  //////////////////////////////
  sendBuffer[7] = (uint8)(0x01);    //Blocknum = 1
  extendedChecksum(sendBuffer, 8);
  
  sentRec = write(fd, sendBuffer, 8);
  if(sentRec < 8)
  {
    if(sentRec == -1)
      goto writeError0;
    else  
      goto writeError1;
  }	

  sentRec= read(fd, recBuffer, 136);

  if(sentRec < 136)
  {
    if(sentRec == -1)
      goto readError0;
    else  
      goto readError1;
  }

  if(recBuffer[1] != (uint8)(0xF8) || recBuffer[2] != (uint8)(0x41) || recBuffer[3] != (uint8)(0x2A))
  {
    goto commandByteError;
  }
  
  caliInfo->bipolarSlope = FPuint8ArrayToFPDouble(recBuffer + 8, 0);
  caliInfo->bipolarOffset = FPuint8ArrayToFPDouble(recBuffer + 8, 8);
  
  //////////////////////////////
  //reading block 2 from memory 
  //////////////////////////////
  sendBuffer[7] = (uint8)(0x02);    //Blocknum = 2
  extendedChecksum(sendBuffer, 8);
  
  sentRec = write(fd, sendBuffer, 8);
 if(sentRec < 8)
  {
    if(sentRec == -1)
      goto writeError0;
    else  
      goto writeError1;
  }	

  sentRec= read(fd, recBuffer, 136);
 if(sentRec < 136)
  {
    if(sentRec == -1)
      goto readError0;
    else  
      goto readError1;
  }

  if(recBuffer[1] != (uint8)(0xF8) || recBuffer[2] != (uint8)(0x41) || recBuffer[3] != (uint8)(0x2A))
  {
    goto commandByteError;
  }
  
  caliInfo->DACSlope[0] = FPuint8ArrayToFPDouble(recBuffer + 8, 0);
  caliInfo->DACOffset[0] = FPuint8ArrayToFPDouble(recBuffer + 8, 8);
  caliInfo->DACSlope[1] = FPuint8ArrayToFPDouble(recBuffer + 8, 16);
  caliInfo->DACOffset[1] = FPuint8ArrayToFPDouble(recBuffer + 8, 24);
  caliInfo->tempSlope = FPuint8ArrayToFPDouble(recBuffer + 8, 32);
  caliInfo->tempSlopeLow = FPuint8ArrayToFPDouble(recBuffer + 8, 48);
  caliInfo->calTemp = FPuint8ArrayToFPDouble(recBuffer + 8, 64);
  caliInfo->Vref = FPuint8ArrayToFPDouble(recBuffer + 8, 72);
  caliInfo->VrefDiv2 = FPuint8ArrayToFPDouble(recBuffer + 8, 88);
  caliInfo->VsSlope = FPuint8ArrayToFPDouble(recBuffer + 8, 96);

  return 0;
  
writeError0: 
  printf("Error : getCalibrationInfo write failed\n");
  return -1;
  
writeError1:
  printf("Error : getCalibrationInfo did not write all of the buffer\n");
  return -1;

readError0:
  printf("Error : getCalibrationInfo read failed\n");
  return -1;
  
readError1:
  printf("Error : getCalibrationInfo did not read all of the buffer\n");
  return -1;

commandByteError:
  printf("Error : received buffer at byte 1, 2, or 3 are not 0xA3, 0x01, 0x2A \n");
  return -1;
}
